;****************************************************************************
; MODULO DE AUDIO - SOUND BLASTER
;****************************************************************************

PUBLIC MASB_INICIA_PROCESADOR
PUBLIC MASB_RECIBE_VALOR
PUBLIC MASB_ENVIA_VALOR

PUBLIC MASB_CONECTA_ALTAVOZ
PUBLIC MASB_DESCONECTA_ALTAVOZ

PUBLIC MASB_COPIA_NIVEL
PUBLIC MASB_PEGA_NIVEL

PUBLIC MASB_ENVIA_SONIDO
PUBLIC MASB_PAUSA_SONIDO
PUBLIC MASB_CONTINUA_SONIDO

PUBLIC MASB_PUERTO_BASE

PUERTO_INICIALIZACION = 006H
PUERTO_LECTURA        = 00EH
PUERTO_ESCRITURA      = 00CH

CAMINO SEGMENT PUBLIC
  ASSUME CS:CAMINO,DS:CAMINO,SS:CAMINO,ES:CAMINO

  DB 'MASB 1998 Marco Antonio Alvarado'

;****************************************************************************
; Inicia el procesador de sonido digital (DSP). Este paso debe ejecutarse
; antes de programarlo.
;
; Regresa:
;   CF = 0, tuvo xito
;      = 1, la tarjeta de sonido no est instalada o el puerto no es el
;          correcto.
;
; Lee:
;   PUERTO_BASE
;****************************************************************************

  LABEL MASB_INICIA_PROCESADOR NEAR
  INICIA_PROCESADOR PROC NEAR
    PUSH DX
    PUSH AX
    PUSH CX

    MOV DX,PUERTO_BASE
    ADD DL,PUERTO_INICIALIZACION
    MOV AL,1
    OUT DX,AL
    SUB AL,AL

  ID_ESPERA:

    DEC AL
    JNZ ID_ESPERA
    OUT DX, AL
    SUB CX, CX

  ID_REPITE:

    MOV DX,PUERTO_BASE
    ADD DL,00EH
    IN AL,DX
    OR AL,AL
    JNS ID_INTENTA
    SUB DL,4
    IN AL,DX
    CMP AL,0AAH
    JE ID_CONTINUA

  ID_INTENTA:

    LOOP ID_REPITE

    STC
    JMP ID_FINALIZA

  ID_CONTINUA:

    CLC

  ID_FINALIZA:

    POP CX
    POP AX
    POP DX
    RET
  INICIA_PROCESADOR ENDP

;****************************************************************************
; Recibe un valor del procesador de sonido.
;
; Regresa:
;   AL = valor
;
; Lee:
;   PUERTO_BASE
;****************************************************************************

  LABEL MASB_RECIBE_VALOR NEAR
  RECIBE_VALOR PROC NEAR
    PUSH DX

    MOV DX,PUERTO_BASE
    ADD DL,PUERTO_LECTURA

  CS_ESPERA:

    IN AL,DX
    OR AL,AL
    JNS CS_ESPERA
    SUB DL,4
    IN AL,DX

    POP DX
    RET
  RECIBE_VALOR ENDP

;****************************************************************************
; Enva un valor al procesador de sonido. Este valor puede ser un comando o
; la informacin requerida.
;
; Necesita:
;   AL = valor
;
; Lee:
;   PUERTO_BASE
;****************************************************************************

  LABEL MASB_ENVIA_VALOR NEAR
  ENVIA_VALOR PROC NEAR
    PUSH AX
    PUSH DX

    MOV AH,AL
    MOV DX,PUERTO_BASE
    ADD DL,PUERTO_ESCRITURA

  PS_ESPERA:

    IN AL,DX
    OR AL,AL
    JS PS_ESPERA
    MOV AL,AH
    OUT DX,AL

    POP DX
    POP AX
    RET
  ENVIA_VALOR ENDP

;****************************************************************************
; Conecta el convertidor analgico-digital (DAC) al altavoz. Este paso es
; necesario para escuchar los sonidos.
;****************************************************************************

  LABEL MASB_CONECTA_ALTAVOZ NEAR
  CONECTA_ALTAVOZ PROC NEAR
    PUSH AX

    MOV AL,0D1H
    CALL ENVIA_VALOR

    POP AX
    RET
  CONECTA_ALTAVOZ ENDP

;****************************************************************************
; Desconecta el convertidor analgico-digital (DAC) del altavoz.
;****************************************************************************

  LABEL MASB_DESCONECTA_ALTAVOZ NEAR
  DESCONECTA_ALTAVOZ PROC NEAR
    PUSH AX

    MOV AL,0D3H
    CALL ENVIA_VALOR

    POP AX
    RET
  DESCONECTA_ALTAVOZ ENDP

;****************************************************************************
; Recibe un nivel del microfono. Un grupo de niveles forma un sonido.
;
; Regresa:
;   AH = nivel de salida
;****************************************************************************

  LABEL MASB_COPIA_NIVEL NEAR
  COPIA_NIVEL PROC NEAR
    XCHG AL,AH

    MOV AL,020H
    CALL ENVIA_VALOR
    CALL RECIBE_VALOR

    XCHG AL,AH
    RET
  COPIA_NIVEL ENDP

;****************************************************************************
; Enva un nivel al altavoz.
;
; Necesita:
;   AH = nivel de salida
;****************************************************************************

  LABEL MASB_PEGA_NIVEL NEAR
  PEGA_NIVEL PROC NEAR
    PUSH AX

    MOV AL,010H
    CALL ENVIA_VALOR
    MOV AL,AH
    CALL ENVIA_VALOR

    POP AX
    RET
  PEGA_NIVEL ENDP

;****************************************************************************
; Enva un sonido a la tarjeta con el acceso directo de memoria (DMA).
; Usa el modo de 8 bits y un solo ciclo.
;
; Requiere:
;   DS:SI = direccin del sonido
;   CX    = cantidad de niveles - 1 (1 nivel = 8 bits)
;   AL    = byte alto de la constante de tiempo. Se calcula como:
;             256-(1000000/(canales*frecuencia))
;             canales    = 1, mono
;                          2, estereo
;             frecuencia = 4000..13000 (Hz)
;
; Bytes altos de la constante de tiempo:
;   0A5H = mono y 11000 Hz
;****************************************************************************

  LABEL MASB_ENVIA_SONIDO NEAR
  ENVIA_SONIDO PROC NEAR
    PUSH BX
    PUSH DX
    PUSH SI
    PUSH AX

    CALL CONECTA_ALTAVOZ

;----------------------------------------------------------------------------
; Calcula el campo.
    MOV BX,DS
    SHL BX,4
    ADD BX,SI

;----------------------------------------------------------------------------
; Calcula la pgina.
    MOV DX,DS
    SHR SI,4
    ADD DX,SI
    SHR DX,12

;----------------------------------------------------------------------------
; Programa el DMA.
    PUSH AX
    MOV AL,5
    OUT 00AH,AL
    MOV AL,0
    OUT 00CH,AL
    MOV AL,049H
    OUT 00BH,AL
    MOV AL,BL
    OUT 002H,AL
    MOV AL,BH
    OUT 002H,AL
    MOV AL,DL
    OUT 083H,AL
    MOV AL,CL
    OUT 003H,AL
    MOV AL,CH
    OUT 003H,AL
    MOV AL,1
    OUT 00AH,AL

;----------------------------------------------------------------------------
; Establece la constante de tiempo de transferencia.
    MOV AL,040H
    CALL ENVIA_VALOR
    POP AX
    CALL ENVIA_VALOR

;----------------------------------------------------------------------------
; Enva el sonido.
    MOV AL,014H
    CALL ENVIA_VALOR
    MOV AL,CL
    CALL ENVIA_VALOR
    MOV AL,CH
    CALL ENVIA_VALOR

    POP AX
    POP SI
    POP DX
    POP BX
    RET
  ENVIA_SONIDO ENDP

;****************************************************************************
; Pausa la transmisin del sonido.
;****************************************************************************

  LABEL MASB_PAUSA_SONIDO NEAR
  PAUSA_SONIDO PROC NEAR
    PUSH AX

    MOV AL,0D0H
    CALL ENVIA_VALOR

    POP AX
    RET
  PAUSA_SONIDO ENDP

;****************************************************************************
; Continua la transmisin del sonido.
;****************************************************************************

  LABEL MASB_CONTINUA_SONIDO NEAR
  CONTINUA_SONIDO PROC NEAR
    PUSH AX

    MOV AL,0D4H
    CALL ENVIA_VALOR

    POP AX
    RET
  CONTINUA_SONIDO ENDP

;****************************************************************************
; Almacena los valores.
;****************************************************************************

  LABEL MASB_PUERTO_BASE WORD
  PUERTO_BASE DW 0220H

CAMINO ENDS

END